
#    MIDAS Httpd Server to provide Data Acquisition Control Services

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

# Use namespaces to isolate our methods
namespace eval urn:DataAcquisitionControlServer {

    proc ServerVerbose {opt} {
        do_ServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc Get_State {} {
        set vl [do_Get_State]
        return [list [list Code int [first $vl]]  [list State string [second $vl]]  [list Reason string [third $vl]]]
    }

    proc Do_Setup {} {
        do_Setup
        return [rpcvar::rpcvar int 0]
    }

    proc Do_Stop {} {
        do_Stop 
        return [rpcvar::rpcvar int 0]
    }

    proc Do_Go {} {
        do_Go
        return [rpcvar::rpcvar int 0]
    }

    proc Get_Counters {} {
        set vl [do_Get_Counters]
        return [list [list GoodEvents int [lindex $vl 0]] [list BadEvents int [lindex $vl 1]] [list InputBuffers int [lindex $vl 2]] [list OutputBuffers int [lindex $vl 3]]]
    }

    proc Get_Rates {} {
        set vl [do_Get_Rates]
        return [list [list GoodEvents int [lindex $vl 0]] [list BadEvents int [lindex $vl 1]] [list InputBuffers int [lindex $vl 2]] [list OutputBuffers int [lindex $vl 3]]]
    }

    proc do_ServerVerbose {option} {
        global MIDAS_DataAcquisitionControlServer_Verbose
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method ServerVerbose with parameter $option"}
        set MIDAS_DataAcquisitionControlServer_Verbose $option
    }

    proc do_Get_State {} {
       global env

       set env(EGsilent) 1
       set z [catch {inform sys Get_State} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Setup {} {

       set z [catch {inform sys do_Setup} m]

       if {$z == 0} then {
          return 0
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Stop {} {

       set z [catch {inform sys do_Stop} m]

       if {$z == 0} then {
          return 0
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Go {} {

       set z [catch {inform sys do_Go} m]

       if {$z == 0} then {
          return 0
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Get_Counters {} {
        global env

       set env(EGsilent) 1
       set z [catch {inform stats Get_Counters} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Get_Rates {} {
       global env

       set env(EGsilent) 1
       set z [catch {inform stats Get_Rates} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }



    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export Get_State
     SOAP::export Do_Setup
     SOAP::export Do_Stop
     SOAP::export Do_Go
     SOAP::export Get_Counters
     SOAP::export Get_Rates
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /DataAcquisitionControlServer -namespace urn:DataAcquisitionControlServer -uri urn:DataAcquisitionControlServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DataAcquisitionControlServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}
