
#    MIDAS Access Server  for  SM   (LyrTech Singal Master) server

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_SM_Verbose
set MIDAS_SM_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

package require SM 1.0


# Use namespaces to isolate your methods
namespace eval urn:SM {

    proc SetUpTemp {} {
        global env tcl_platform

        if {![info exists env(MIDASTempDir)] || ![file exists $env(MIDASTempDir)]} then {
           switch $tcl_platform(platform) {
              windows {set env(MIDASTempDir) [file join c:/ temp]}
              default {set env(MIDASTempDir) \tmp}
           }
        }

        catch {file mkdir [file join $env(MIDASTempDir) SMServer]}
    }

    proc SMVerbose {vl} {
        global MIDAS_SM_Verbose MIDAS_SOAPDomainHandler_Verbose

        set MIDAS_SM_Verbose $vl
        set MIDAS_SOAPDomainHandler_Verbose $vl

        return [rpcvar::rpcvar int 0]
    }

    proc SMInit {} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMInit"}

        set rc [SM_Init]
        SetUpTemp

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc SMUpLoad {file contents} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMUpLoad"}

        set path [file join $env(MIDASTempDir) SMServer [file tail $file].base64]

        set z [catch {open $path w} f]
        if {$z != 0} then {error "Cannot create/open file $path in SMUpLoad\n$f"}

        puts $f $contents
        close $f

        set f [file join $env(MIDASTempDir) SMServer [file tail $file]]

        set z [catch {exec base64 --decode $path $f} m]
        if {$z != 0} then {error "Error in base64 decoding $path in SMUpLoad\n$m"}

        set rc 0

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc SMGetMode {id} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMGetMode"}

        set rc [SM_Get $id 0x00001]     ;# SM_GET_MODE

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc SMGetSMType {id} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMGetSMType"}

        set rc [SM_Get $id 0x10002]     ;# SM_GET_SMTYPE

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc SMGetVersion {id component} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMGetVersion"}

        set rc [SM_Get $id 0x10003 $component]     ;# SM_GET_VERSION

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMGetUsers {id} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMGetUsers"}

        set rc [SM_Get $id 0x10201]     ;# SM_GET_USERS

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetUser {id user} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetUsers"}

        set rc [SM_Set $id 0x10202 $user]     ;# SM_SET_USER

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetLock {id lock} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetLock"}

        set rc [SM_Set $id 0x00002 $lock]     ;# SM_SET_LOCK

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMGetLock {id} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMGetLock"}

        set rc [SM_Get $id 0x00003]     ;# SM_GET_LOCK

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMGetDSPRead {id offset len} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMGetDSPRead"}

        set rc [SM_Get $id 0x20012 $offset $len]     ;# SM_GET_DSP_READ

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetDSPWrite {id offset len data} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetDSPWrite"}

        set rc [SM_Set $id 0x20011 $offset $len $data]     ;# SM_SET_DSP_WRITE

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetFPGALoad {id bitstream} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetFPGALoad"}

        set path [file join $env(MIDASTempDir) SMServer [file tail $bitstream]]

        set rc [SM_Set $id 0x70001 $path]     ;# SM_SET_FPGABITSTREAM

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetFPGAFlash {id bitstream} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetFPGAFlash"}

        set path [file join $env(MIDASTempDir) SMServer [file tail $bitstream]]

        set rc [SM_Set $id 0x30003 $path]     ;# SM_SET_FLASHFPGA

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetDSPFlash {id format filename} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetDSPFlash"}

        set path [file join $env(MIDASTempDir) SMServer [file tail $filename]]

        set rc [SM_Set $id 0x30002 $format $path]     ;# SM_SET_FLASHDSP

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetDSPReset {id select} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetDSPReset"}

        set rc [SM_Set $id 0x20020 $select]     ;# SM_SET_MDSP_RESET

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMSetDSPLoad {id select format filename} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMSetDSPLoad"}

        set path [file join $env(MIDASTempDir) SMServer [file tail $filename]]

        set rc [SM_Set $id 0x20021 $select $format $path]     ;# SM_SET_MDSP_LOAD

        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc SMExit {} {
        global env MIDAS_SM_Verbose

        if {$MIDAS_SM_Verbose} \
             {puts "called method SM::SMExit"}

        set rc 0
             
        if {$MIDAS_SM_Verbose}  {puts "SM reply $rc"}

        return [rpcvar::rpcvar int 0]
    }


    # We have to publish the public methods...

     SOAP::export SMVerbose
     SOAP::export SMInit


     SOAP::export SMExit
     
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /SM -namespace urn:SM -uri urn:SM]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/SM {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

