
#    MIDAS Access Server  for  VHSADC server

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_VHSADC_Verbose
set MIDAS_VHSADC_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

package require VHSADC 1.0


# Use namespaces to isolate your methods
namespace eval urn:VHSADC {


    proc SetUpTemp {} {
        global env tcl_platform

        if {![info exists env(MIDASTempDir)] || ![file exists $env(MIDASTempDir)]} then {
           switch $tcl_platform(platform) {
              windows {set env(MIDASTempDir) [file join c:/ temp]}
              default {set env(MIDASTempDir) \tmp}
           }
        }   

        catch {file mkdir $env(MIDASTempDir}
        catch {file mkdir [file join $env(MIDASTempDir) VHSADCServer]}
    }

    proc VHSADCVerbose {vl} {
        global MIDAS_VHSADC_Verbose MIDAS_SOAPDomainHandler_Verbose

        set MIDAS_VHSADC_Verbose $vl
        set MIDAS_SOAPDomainHandler_Verbose $vl
        VHSADC_Verbose  $vl

        return [rpcvar::rpcvar int 0]
    }

    proc VHSADCInit {} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADCInit"}

        set rc [VHSADC_Init]
        SetUpTemp

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCUpLoad {file contents} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_UpLoad"}

        set path [file join $env(MIDASTempDir) VHSADCServer [file tail $file].base64]

        set z [catch {open $path w} f]
        if {$z != 0} then {error "Cannot create/open file $path in VHSADCUpLoad\n$f"}

        puts $f $contents
        close $f

        set f [file join $env(MIDASTempDir) VHSADCServer [file tail $file]]

        set z [catch {exec base64 --decode $path $f} m]
        if {$z != 0} then {error "Error in base64 decoding $path in VHSADCUpLoad\n$m"}

        set rc 0

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetAcquisitionStatus {id status} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetAcquisitionStatus"}

        set rc [VHSADC_SetAcquisitionStatus $id $status]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCGetFPGARevision {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_GetFPGARevision"}

        set rc [VHSADC_GetFPGARevision $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetLed {id number type state} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetLed"}

        set rc [VHSADC_SetLed $id $number $type $state]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetAcquisitionCLK {id source} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetAcquisitionCLK"}

        set rc [VHSADC_SetAcquisitionCLK $id $source]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetTriggerOption {id mode} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetTriggerOption"}

        set rc [VHSADC_SetTriggerOption $id $mode]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetGainEnableMode {id channel} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetGainEnableMode"}

        set rc [VHSADC_SetGainEnableMode $id $channel]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetGainLevel {id ctrl level} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetGainLevel"}

        set rc [VHSADC_SetGainLevel $id $ctrl $level]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCGetGainLevelReg {id level} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_GetGainLevelReg"}

        set rc [VHSADC_GetGainLevelReg $id $level]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetGainMode {id channel} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetGainMode"}

        set rc [VHSADC_SetGainMode $id $channel]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCSetUserCtrlReg {id number value} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_SetUserCtrlReg"}

        set rc [VHSADC_SetUserCtrlReg $id $number $value]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc VHSADCGetUserCtrlReg {id number} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::VHSADC_GetUserCtrlReg"}

        set rc [VHSADC_GetUserCtrlReg $id $number]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16FpgaReset {id bitstream len} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_FpgaReset"}

        set path [file join $env(MIDASTempDir) VHSADCServer [file tail $bitstream]]

        set rc [Vhsadac16_FpgaReset $id $path $len]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16FpgaLoad {id bitstream} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_FpgaLoad"}

        set path [file join $env(MIDASTempDir) VHSADCServer [file tail $bitstream]]

        set rc [Vhsadac16_FpgaLoad $id $path]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16ReadFpgaReg {id register} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_ReadFpgaReg"}

        set rc [Vhsadac16_ReadFpgaReg $id $register]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16WriteFpgaReg {id register value} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_WriteFpgaReg"}

        set rc [Vhsadac16_WriteFpgaReg $id $register $value]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16WriteFpgaMem {id offset len data} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_WriteFpgaMem"}

        set rc [Vhsadac16_WriteFpgaMem $id $offset $len $data]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16ReadFpgaMem {id offset len} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_ReadFpgaMem"}

        set rc [Vhsadac16_ReadFpgaMem $id $offset $len]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc Vhsadac16GetCpldRegs {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_GetCpldRegs"}

        set rc [Vhsadac16_GetCpldRegs $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc Vhsadac16Sdram_AutoInit {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_AutoInit"}

        set rc [Vhsadac16_Sdram_AutoInit $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Clock_Frequency {id freq} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Clock_Frequency"}

        set rc [Vhsadac16_Clock_Frequency $id $freq]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Sdram_Frequency {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_Frequency"}

        set rc [Vhsadac16_Sdram_Frequency $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Sdram_Size {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_Size"}

        set rc [Vhsadac16_Sdram_Size $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Sdram_SinglePageMode {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_SinglePageMode"}

        set rc [Vhsadac16_Sdram_SinglePageMode $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Sdram_DualPageMode {id} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_DualPageMode"}

        set rc [Vhsadac16_Sdram_DualPageMode $id]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Sdram_Write {id offset len data} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_Write"}

        set rc [Vhsadac16_Sdram_Write $id $offset $len $data]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int $rc]
    }

    proc Vhsadac16Sdram_Read {id offset len} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::Vhsadac16_Sdram_Read"}

        set rc [Vhsadac16_Sdram_Read $id $offset $len]

        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar string $rc]
    }

    proc VHSADCExit {} {
        global env MIDAS_VHSADC_Verbose

        if {$MIDAS_VHSADC_Verbose} \
             {puts "called method VHSADC::ServerExit"}

   set rc 0
             
        if {$MIDAS_VHSADC_Verbose}  {puts "VHSADC reply $rc"}

        return [rpcvar::rpcvar int 0]
    }


    # We have to publish the public methods...

     SOAP::export VHSADCVerbose
     SOAP::export VHSADCInit

     SOAP::export VHSADCUpLoad

     SOAP::export VHSADCSetAcquisitionStatus
     SOAP::export VHSADCGetFPGARevision
     SOAP::export VHSADCSetLed
     SOAP::export VHSADCSetAcquisitionCLK
     SOAP::export VHSADCSetTriggerOption
     SOAP::export VHSADCSetGainEnableMode
     SOAP::export VHSADCSetGainLevel
     SOAP::export VHSADCGetGainLevelReg
     SOAP::export VHSADCSetGainMode
     SOAP::export VHSADCSetUserCtrlReg
     SOAP::export VHSADCGetUserCtrlReg

     SOAP::export Vhsadac16FpgaReset
     SOAP::export Vhsadac16FpgaLoad

     SOAP::export Vhsadac16ReadFpgaReg
     SOAP::export Vhsadac16WriteFpgaReg

     SOAP::export Vhsadac16ReadFpgaMem
     SOAP::export Vhsadac16WriteFpgaMem

     SOAP::export Vhsadac16GetCpldRegs
     SOAP::export Vhsadac16Clock_Frequency

     SOAP::export Vhsadac16Sdram_AutoInit
     SOAP::export Vhsadac16Sdram_Frequency
     SOAP::export Vhsadac16Sdram_Size
     SOAP::export Vhsadac16Sdram_SinglePageMode
     SOAP::export Vhsadac16Sdram_DualPageMode

     SOAP::export Vhsadac16Sdram_Write
     SOAP::export Vhsadac16Sdram_Read

     SOAP::export VHSADCExit
     
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /VHSADC -namespace urn:VHSADC -uri urn:VHSADC]


#proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/VHSADC {optsname sock args} {   \
#    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

